import acm.graphics.*;
import acm.program.*;
import java.awt.*;

/**
 * A class representing a Papert-style Turtle that can be moved about a canvas.
 * This class acts as an initializer and wrapper for the JTF Turtle, so that it 
 * can be controlled through the BlueJ interface
 * 
 * Note that this class uses a singleton canvas (run as an apple
 * 
 * @author Joel
 * @version Aug 2013
 */
public class Turtle
{
    private GTurtle turtle; //the underlying turtle object

    private static TurtleCanvas canvas = null; //another singleton canvas
    
    /**
     * Constructor for objects of class Turtle
     */
    public Turtle()
    {
        turtle = new GTurtle(); //our turtle object is created here!
        turtle.setSize(30); //make the turtle not too big
        turtle.penDown(); //start with pen down
        
        if(canvas == null)
        {
            canvas = new TurtleCanvas();
            canvas.start();
        }
        canvas.addTurtle(turtle);
    }
    
    /**
     * Moves the turtle forward by one "step" (the size of the turtle; 30 pixels).
     */
    public void forward() {
        turtle.forward();
        turtle.setColor(turtle.getColor()); //fix for lines not showing up via BlueJ...
    }
    
    /**
     * Moves the turtle forward the specified distance (in pixels)
     */
    public void forward(double distance) {
        turtle.forward(distance);
        turtle.setColor(turtle.getColor()); //fix for lines not showing up via BlueJ...
    }
    
    /**
     * Gets the turtle's current facing (in degrees counterclockwise from the x axis)
     */
    public double getDirection() {
        return turtle.getDirection();
    }
    
    /**
     * Gets the turtle's current speed. 0.5 is default, 1 is fastest, 0 is slowest.
     */
    public double getSpeed() {
        return turtle.getSpeed();
    }
    
    /**
     * Gets whether the turtle's pen is down or not.
     */
    public boolean isPenDown() {
        return turtle.isPenDown();
    }
    
    /**
     * Turns the turtle to the left the specified number of degrees
     */
    public void left(double angle) {
        turtle.left(angle);
    }
    
    /**
     * Puts the turtle's pen down.
     */
    public void penDown() {
        turtle.penDown();
    }
    
    /**
     * Lifts the turtle's pen up.
     */
    public void penUp() {
        turtle.penUp();
    }
    
    /**
     * Prints the given message on the screen. Note that this will not move the turtle.
     */
    public void printText(String message) {
        turtle.printText(message);
        turtle.setColor(turtle.getColor());
    }
    
    /**
     * Turns the turtle to the right the specified number of degrees
     */
    public void right(double angle) {
        turtle.right(angle);
    }
    
    /**
     * Sets the color of the turtle's pen.
     * Note that if called through BlueJ, you will need to specify the full path to the color
     * (i.e., java.awt.Color.RED or java.awt.Color.BLUE).
     */
    public void setPenColor(Color color) {
        turtle.setColor(color);
    }
    
    /**
     * Sets the color of the turtle's pen, using the name of the color. Can either be:
     * "red", "green", "blue", "cyan", "magenta", "yellow", "pink", "gray", "black", "white"
     */
    public void setPenColorByName(String color) {
        turtle.setColor(getColorFromString(color));
    }
    
    /**
     * Sets the turtle's facing angle, in degrees counterclockwise from the x-axis
     */
    private void setDirection(double dir) {
        turtle.setDirection(dir);
    }
    
    /**
     * Sets the turtle's location on the screen
     */
    private void setLocation(double x, double y) {
        turtle.setLocation(x,y);
    }
    
    /**
     * Sets the turtle's speed. 0.5 is default, 1 is fastest, 0 is slowest.
     */
    public void setSpeed(double speed) {
        turtle.setSpeed(speed);
    }
    
    /**
     * Returns the color of the turtle's pen.
     * Note that this will look like jibberish in BlueJ.
     */
    public Color getPenColor() {
        return turtle.getColor();
    }
    
    /**
     * Returns the name of the color of the turtle's pen.
     */
    public String getPenColorName() {
        Color c = turtle.getColor();
        if(c == Color.BLUE)
            return "blue";
        if(c == Color.CYAN)
            return "cyan";
        if(c == Color.GRAY)
            return "gray";
        if(c == Color.GREEN)
            return "green";
        if(c == Color.MAGENTA)
            return "magenta";
        if(c == Color.ORANGE)
            return "orange";
        if(c == Color.PINK)
            return "pink";
        if(c == Color.RED)
            return "red";
        if(c == Color.WHITE)
            return "white";
        if(c == Color.YELLOW)
            return "yellow";
        if(c == Color.BLACK)
            return "black";
        return "Unknown color";
    }
    
    /**
     * Helper method for converting Strings to colors
     */
    private Color getColorFromString(String c)
    {
        if(c.equals("blue"))
            return Color.BLUE;
        if(c.equals("cyan"))
            return Color.CYAN;
        if(c.equals("gray"))
            return Color.GRAY;
        if(c.equals("green"))
            return Color.GREEN;
        if(c.equals("magenta"))
            return Color.MAGENTA;
        if(c.equals("orange"))
            return Color.ORANGE;
        if(c.equals("pink"))
            return Color.PINK;
        if(c.equals("red"))
            return Color.RED;
        if(c.equals("white"))
            return Color.WHITE;
        if(c.equals("yellow"))
            return Color.YELLOW;
        return Color.BLACK; //return black otherwise
    }
        
        
    /**
     * A private class that controls the window; basically just adds the turtle
     */
    private class TurtleCanvas extends GraphicsProgram
    {
        public void run() 
        {
            this.setTitle("Turtle Drawing");
        }
        
        public void addTurtle(GTurtle turtle)
        {
            this.add(turtle, 100, getHeight()-100);
        }
    }
}
